﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;
using System.Windows.Threading;

namespace SharpGIS.MouseExtensions
{
	public static class DoubleClick
	{
		private const int doubleClickInterval = 200;
		private static readonly DependencyProperty DoubleClickTimerProperty = DependencyProperty.RegisterAttached("DoubleClickTimer", typeof(DispatcherTimer), typeof(DoubleClick), null);
		private static readonly DependencyProperty DoubleClickHandlersProperty = DependencyProperty.RegisterAttached("DoubleClickHandlers", typeof(IList<MouseButtonEventHandler>), typeof(DoubleClick), null);
		private static readonly DependencyProperty DoubleClickPositionProperty = DependencyProperty.RegisterAttached("DoubleClickPosition", typeof(Point), typeof(DoubleClick), null);
		/// <summary>
		/// Adds a double click event handler.
		/// </summary>
		/// <param name="element">The Element to listen for double clicks on.</param>
		/// <param name="handler">The handler.</param>
		public static void AttachDoubleClick(this UIElement element, MouseButtonEventHandler handler)
		{
			IList<MouseButtonEventHandler> handlers = element.GetValue(DoubleClickHandlersProperty) as IList<MouseButtonEventHandler>;
			if (handlers == null)
			{
				element.MouseLeftButtonDown += element_MouseLeftButtonDown;
				handlers = new List<MouseButtonEventHandler>();
				element.SetValue(DoubleClickHandlersProperty, handlers);
			}
			handlers.Add(handler);
		}
		/// <summary>
		/// Removes a double click event handler.
		/// </summary>
		/// <param name="element">The element.</param>
		/// <param name="handler">The handler.</param>
		public static void DetachDoubleClick(this UIElement element, MouseButtonEventHandler handler)
		{
			IList<MouseButtonEventHandler> handlers = element.GetValue(DoubleClickHandlersProperty) as IList<MouseButtonEventHandler>;
			if (handlers != null)
			{
				if (handlers.Contains(handler))
				{
					handlers.Remove(handler);
					if (handlers.Count == 0)
					{
						element.ClearValue(DoubleClickHandlersProperty);
						element.MouseLeftButtonDown -= element_MouseLeftButtonDown;
					}
				}
			}
		}
		private static void element_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			UIElement element = sender as UIElement;
			Point position = e.GetPosition(element);
			DispatcherTimer timer = element.GetValue(DoubleClickTimerProperty) as DispatcherTimer;
			if (timer != null) //DblClick
			{
				timer.Stop();
				Point oldPosition = (Point)element.GetValue(DoubleClickPositionProperty);
				element.ClearValue(DoubleClickTimerProperty);
				element.ClearValue(DoubleClickPositionProperty);
				if (Math.Abs(oldPosition.X - position.X) < 1 && Math.Abs(oldPosition.Y - position.Y) < 1) //mouse didn't move => Valid double click
				{
					IList<MouseButtonEventHandler> handlers = element.GetValue(DoubleClickHandlersProperty) as IList<MouseButtonEventHandler>;
					if (handlers != null)
					{
						foreach (MouseButtonEventHandler handler in handlers)
						{
							handler(sender, e);
						}
					}
					return;
				}
			}
			//First click or mouse moved. Start a new timer
			timer = new DispatcherTimer() { Interval = TimeSpan.FromMilliseconds(doubleClickInterval) };
			timer.Tick += new EventHandler((s, args) =>
			{  //DblClick timed out
				(s as DispatcherTimer).Stop();
				element.ClearValue(DoubleClickTimerProperty); //clear timer
				element.ClearValue(DoubleClickPositionProperty); //clear first click position
			});
			element.SetValue(DoubleClickTimerProperty, timer);
			element.SetValue(DoubleClickPositionProperty, position);
			timer.Start();
		}
	}
}
