﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.Windows.Browser;

namespace SharpGIS.MouseExtensions
{
    public sealed class MouseWheelEventArgs : EventArgs
    {
        internal MouseWheelEventArgs(double delta, Point p)
        {
            Delta = delta;
            point = p;
        }
        private Point point;
        public double Delta { get; private set; }
        public bool Handled { get; set; }
        public Point GetPosition(UIElement relativeTo)
        {
            return Application.Current.RootVisual.TransformToVisual(relativeTo).Transform(point);
        }
    }

	public static class Wheel
	{
        private static readonly DependencyProperty WheelHandlersProperty = DependencyProperty.RegisterAttached("WheelHandlers", typeof(IList<EventHandler<MouseWheelEventArgs>>), typeof(Wheel), null);
        private static UIElement currentElement;
        public static void AttachWheel(this UIElement element, EventHandler<MouseWheelEventArgs> handler)
		{
            if (listener == null)
            {
                listener = new WheelMouseListener();
                listener.Wheel += listener_Wheel;
            }
            element.MouseEnter += element_MouseEnter;
            element.MouseLeave += element_MouseLeave;
            IList<EventHandler<MouseWheelEventArgs>> handlers = element.GetValue(WheelHandlersProperty) as IList<EventHandler<MouseWheelEventArgs>>;
            if (handlers == null)
            {
                handlers = new List<EventHandler<MouseWheelEventArgs>>();
                element.SetValue(WheelHandlersProperty, handlers);
            }
            handlers.Add(handler);
        }

        public static void DetachWheel(this UIElement element, EventHandler<MouseWheelEventArgs> handler)
        {
            element.MouseEnter -= element_MouseEnter;
            element.MouseLeave -= element_MouseLeave;
            IList<EventHandler<MouseWheelEventArgs>> handlers = element.GetValue(WheelHandlersProperty) as IList<EventHandler<MouseWheelEventArgs>>;
            if (handlers != null)
            {
                if (handlers.Contains(handler))
                {
                    handlers.Remove(handler);
                    if (handlers.Count == 0)
                        element.ClearValue(WheelHandlersProperty);
                }
            }
        }

        private static void element_MouseLeave(object sender, MouseEventArgs e)
        {
            currentElement = null;
        }

        private static void element_MouseEnter(object sender, MouseEventArgs e)
        {
            currentElement = sender as UIElement;
        }

        private static void listener_Wheel(object sender, MouseWheelEventArgs e)
        {
            if (currentElement != null)
            {
                  IList<EventHandler<MouseWheelEventArgs>> handlers = currentElement.GetValue(WheelHandlersProperty) as IList<EventHandler<MouseWheelEventArgs>>;
                  if (handlers != null)
                  {
                      foreach (EventHandler<MouseWheelEventArgs> handler in handlers)
                          handler(currentElement, e);
                  }
            }
        }

        private static WheelMouseListener listener = null;

        private class WheelMouseListener
        {
            public event EventHandler<MouseWheelEventArgs> Wheel;
            internal WheelMouseListener()
            {
                if (HtmlPage.IsEnabled)
                {
					if (HtmlPage.BrowserInformation.Name == "Microsoft Internet Explorer")
					{
						HtmlPage.Plugin.AttachEvent("mousewheel", this.Plugin_MouseWheel);
					}
					else
					{
						HtmlPage.Window.AttachEvent("mousewheel", this.Plugin_MouseWheel);
						if (!HtmlPage.BrowserInformation.UserAgent.Contains("Chrome"))
						{
							HtmlPage.Window.AttachEvent("DOMMouseScroll", this.Plugin_MouseWheel);
						}
					}
                }
            }
            private void Plugin_MouseWheel(object sender, HtmlEventArgs args)
            {
                double mouseDelta = 0;
                ScriptObject e = args.EventObject;
                object detail = e.GetProperty("detail");
                object wheelDelta = e.GetProperty("wheelDelta");
				if (wheelDelta != null) // IE, Opera and Chrome
                {
                    mouseDelta = ((double)wheelDelta);
                    if (HtmlPage.Window.GetProperty("opera") != null)
                        mouseDelta *= -1;
                }
                // Mozilla and Safari
                else if (detail != null)
                {
                    mouseDelta = -(double)detail;
                }
                if (mouseDelta != 0 && this.Wheel != null)
                {
                    MouseWheelEventArgs wheelArgs = new MouseWheelEventArgs(Math.Sign(mouseDelta), new Point(args.OffsetX, args.OffsetY));
                    this.Wheel(this, wheelArgs);
                    if (wheelArgs.Handled)
                        args.PreventDefault();
                }
            }
        }
	}
}
